from abaqusConstants import *


class Ratios:
    """The Ratios object specifies ratios that define anisotropic swelling.

    Notes
    -----
    This object can be accessed by:

    .. code-block:: python

        import material
        mdb.models[name].materials[name].moistureSwelling.ratios
        mdb.models[name].materials[name].swelling.ratios
        import odbMaterial
        session.odbs[name].materials[name].moistureSwelling.ratios
        session.odbs[name].materials[name].swelling.ratios

        The table data for this object are:
            - r11.
            - r22.
            - r33.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

    The corresponding analysis keywords are:
        - RATIOS

    """

    def __init__(self, table: tuple, temperatureDependency: Boolean = OFF, dependencies: int = 0):
        """This method creates a Ratios object.

        Notes
        -----
        This function can be accessed by:

        .. code-block:: python

            mdb.models[name].materials[name].moistureSwelling.Ratios
            mdb.models[name].materials[name].swelling.Ratios
            session.odbs[name].materials[name].moistureSwelling.Ratios
            session.odbs[name].materials[name].swelling.Ratios
        
        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below. 
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF. 
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0. 

        Returns
        -------
            A Ratios object. 

        Raises
        ------
        RangeError
        """
        pass

    def setValues(self):
        """This method modifies the Ratios object.

        Raises
        ------
        RangeError
        """
        pass



class UserOutputVariables:
    """The UserOutputVariables object specifies the number of user-defined output variables.

    Notes
    -----
        This object can be accessed by:
        
        .. code-block:: python
            
            import material
            mdb.models[name].materials[name].userOutputVariables
            import odbMaterial
            session.odbs[name].materials[name].userOutputVariables

    The corresponding analysis keywords are:
        - USER OUTPUT VARIABLES

    """

    def __init__(self, n: int = 0):
        """This method creates a UserOutputVariables object.

        Notes
        -----
            This function can be accessed by:
            
            .. code-block:: python
            
                mdb.models[name].materials[name].UserOutputVariables
                session.odbs[name].materials[name].UserOutputVariables
        
        Parameters
        ----------
        n
            An Int specifying the number of user-defined variables required at each material point. 
            The default value is 0. 

        Returns
        -------
            A UserOutputVariables object. 

        Raises
        ------
        RangeError
        """
        pass

    def setValues(self):
        """This method modifies the UserOutputVariables object.

        Raises
        ------
        RangeError
        """
        pass



from abaqusConstants import *


class Regularization:
    """The Regularization object defines the tolerance to be used for regularizing material
    data. 

    Notes
    -----
    This object can be accessed by:

    .. code-block:: python

        import material
        mdb.models[name].materials[name].regularization
        import odbMaterial
        session.odbs[name].materials[name].regularization

    The corresponding analysis keywords are:
        - DASHPOT

    """

    def __init__(self, rtol: float = 0, strainRateRegularization: SymbolicConstant = LOGARITHMIC):
        """This method creates a Regularization object.

        Notes
        -----
        This function can be accessed by:

        .. code-block:: python

            mdb.models[name].materials[name].Regularization
            session.odbs[name].materials[name].Regularization
        
        Parameters
        ----------
        rtol
            A Float specifying the tolerance to be used for regularizing material data. The default 
            value is 0.03. 
        strainRateRegularization
            A SymbolicConstant specifying the form of regularization of strain-rate-dependent 
            material data. Possible values are LOGARITHMIC and LINEAR. The default value is 
            LOGARITHMIC. 

        Returns
        -------
            A Regularization object. 

        Raises
        ------
        RangeError
        """
        pass

    def setValues(self):
        """This method modifies the Regularization object.

        Raises
        ------
        RangeError
        """
        pass



from cpfemConstants import *


class UserMaterial:
    """The UserMaterial object defines material constants for use in subroutines UMAT, UMATHT,
    or VUMAT. 

    Notes
    -----
        This object can be accessed by:
        
        .. code-block:: python
            
            import material
            mdb.models[name].materials[name].userMaterial
            import odbMaterial
            session.odbs[name].materials[name].userMaterial

    The corresponding analysis keywords are:
        - USER MATERIAL

    """

    def __init__(self, type: SymbolicConstant = MECHANICAL, unsymm: Boolean = OFF,
                 mechanicalConstants: tuple = (), thermalConstants: tuple = (), effmod: Boolean = OFF,
                 hybridFormulation: SymbolicConstant = INCREMENTAL):
        """This method creates a UserMaterial object.

        Notes
        -----
            This function can be accessed by:
            
            .. code-block:: python
            
                mdb.models[name].materials[name].UserMaterial
                session.odbs[name].materials[name].UserMaterial
        
        Parameters
        ----------
        type
            A SymbolicConstant specifying the type of material behavior defined by the command. 
            Possible values are MECHANICAL, THERMAL, and THERMOMECHANICAL. The default value is 
            MECHANICAL. 
        unsymm
            A Boolean specifying if the material stiffness matrix, ∂Δσ/∂Δε, is not symmetric or, 
            when a thermal constitutive model is used, if ∂f/∂(∂θ/∂x) is not symmetric. The default 
            value is OFF. This argument is valid only for an Abaqus/Standard analysis. 
        mechanicalConstants
            A sequence of Floats specifying the mechanical constants of the material. This argument 
            is valid only when *type*=MECHANICAL or THERMOMECHANICAL. The default value is an empty 
            sequence. 
        thermalConstants
            A sequence of Floats specifying the thermal constants of the material. This argument is 
            valid only when *type*=THERMAL or THERMOMECHANICAL. The default value is an empty 
            sequence. 
        effmod
            A Boolean specifying if effective bulk modulus and shear modulus are returned by user 
            subroutine VUMAT. The default value is OFF. This argument is valid only in an 
            Abaqus/Explicit analysis. 
        hybridFormulation
            A SymbolicConstant to specify the formulation of the hybrid element with user subroutine 
            UMAT. Possible values are TOTAL, INCREMENTAL, and INCOMPRESSIBLE. The default value is 
            INCREMENTAL. This argument is valid only in an Abaqus/Standard analysis. 

        Returns
        -------
            A UserMaterial object. 

        Raises
        ------
        RangeError
        """
        pass

    def setValues(self):
        """This method modifies the UserMaterial object.

        Raises
        ------
        RangeError
        """
        pass



# -*- coding: utf-8 -*-


class EulerAngle:
    """The EulerAngle object defines Euler angle of a grain.

    Attributes
    ----------
    phi1: float
        1st Euler angle
    Phi: float
        2nd Euler angle
    phi2: float
        3rd Euler angle

    Notes
    -----

    """
    phi1: float = 0.
    Phi: float = 0.
    phi2: float = 0.

    def __init__(self, phi1: float = 0., Phi: float = 0., phi2: float = 0.):
        """The method creates a EulerAngle object.

        Parameters
        ----------
        phi1: float
            1st Euler angle
        Phi: float
            2nd Euler angle
        phi2: float
            3rd Euler angle

        Notes
        -----

        Returns
        -------
            A EulerAngle object.

        Raises
        ------

        """
        self.phi1 = float(phi1)
        self.Phi = float(Phi)
        self.phi2 = float(phi2)

    def setValues(self, phi1: float = 0., Phi: float = 0., phi2: float = 0.):
        """This method modifies the EulerAngle object.

        Parameters
        ----------
        phi1: float
            1st Euler angle
        Phi: float
            2nd Euler angle
        phi2: float
            3rd Euler angle

        """
        self.phi1 = float(phi1)
        self.Phi = float(Phi)
        self.phi2 = float(phi2)

    def __str__(self):
        return "{'phi1': %s, 'Phi': %s, 'phi2': %s}" % (self.phi1, self.Phi, self.phi2)



# -*- coding: utf-8 -*-
import numpy as np
try:
    from abaqus import *
    from abaqusConstants import *
    from caeModules import *
    from driverUtils import executeOnCaeStartup
except ImportError:
    print("Without abaqus environment.")


def create_sketch_base(model_name='Model-1', width=1, height=1, center=[0, 0]):
    # =============================================================================
    # Sketch-Base
    # =============================================================================
    s = mdb.models[model_name].ConstrainedSketch(name='Sketch-Base',
                                                 sheetSize=200.0)
    s.rectangle(
        point1=(
            center[0] - width / 2,
            center[1] - height / 2),
        point2=(
            center[0] + width / 2,
            center[1] + height / 2))


def create_part(model_name='Model-1', part_name='PART-1', dimension=2, pdepth=0,
                pwidth=1, pheight=1, datumpointsfilename='', edgesfilename=''):
    # =============================================================================
    # create base shell
    # =============================================================================
    if dimension == 2:
        p = mdb.models[model_name].Part(name=part_name,
                                        dimensionality=TWO_D_PLANAR,
                                        type=DEFORMABLE_BODY)
        p.BaseShell(sketch=mdb.models[model_name].sketches['Sketch-Base'])
    elif dimension == 3:
        p = mdb.models[model_name].Part(name=part_name,
                                        dimensionality=THREE_D,
                                        type=DEFORMABLE_BODY)
        p.BaseSolidExtrude(
            sketch=mdb.models[model_name].sketches['Sketch-Base'], depth=pdepth)

    # =============================================================================
    # create grains
    # =============================================================================
    # read external datum points file and add datum points to cae model
    fid = open(datumpointsfilename, "r")
    # arrays of x, y, z coordinates of vertices
    arrx = np.zeros(shape=(0))
    arry = np.zeros(shape=(0))
    count = 1
    for line in fid:
        tempdata = line.split()
        # get x and y coordinate of datum point
        tempx = float(tempdata[1])
        tempy = float(tempdata[2])
        arrx = np.append(arrx, tempx)
        arry = np.append(arry, tempy)
        p.DatumPointByCoordinate(coords=(tempx, tempy, pdepth))
        count = count + 1
    fid.close()
    f = p.faces

    # define boundingbox to pick the entire upper face
    BoundBox1 = np.zeros(shape=(3))
    BoundBox2 = np.zeros(shape=(3))
    BoundBox1[0] = -0.1*pwidth
    BoundBox1[1] = -0.1*pheight
    BoundBox1[2] = 0.9*pdepth
    BoundBox2[0] = 1.1*pwidth
    BoundBox2[1] = 1.1*pheight
    BoundBox2[2] = 1.1*pdepth
    pickedFaces = f.getByBoundingBox(
        BoundBox1[0], BoundBox1[1], BoundBox1[2], BoundBox2[0], BoundBox2[1], BoundBox2[2])
    v, e, d = p.vertices, p.edges, p.datums
    # array that contains point at the boundary of edges
    # using .tess file numbering
    edgepoint1 = np.array([], dtype='uint64')
    edgepoint2 = np.array([], dtype='uint64')
    # array with the edge indices
    # counting only edges defined on the upper surface
    # and leaving 0 the ones on the corner edges
    edgeindex = np.array([], dtype='uint64')

    # read external edges file to cae model
    fied = open(edgesfilename, "r")
    count = 1
    for line in fied:
        InnerEdge = True  # flag to indicate that this is not an edge at the boundary of the surface
        f = p.faces
        pickedFaces = f.getByBoundingBox(
            BoundBox1[0], BoundBox1[1], BoundBox1[2], BoundBox2[0], BoundBox2[1], BoundBox2[2])
        v, e, d = p.vertices, p.edges, p.datums
        tempdata = line.split()
        # get indices of the datum points bounding the edge
        # indices start from 1
        # these are indices in the .tess file, not abaqus
        # corresponds to abaqus datum point number - 1
        temppoint1 = int(tempdata[1])
        # corresponds to abaqus datum point number - 1
        temppoint2 = int(tempdata[2])
        edgepoint1 = np.append(edgepoint1, np.uint64(temppoint1))
        edgepoint2 = np.append(edgepoint2, np.uint64(temppoint2))
        # is this an edge at the boundary?
        if (arrx[temppoint1-1] == arrx[temppoint2-1]):
            if (arrx[temppoint1-1] == 0.0 or arrx[temppoint1-1] == pwidth):
                InnerEdge = False
        if (arry[temppoint1-1] == arry[temppoint2-1]):
            if (arry[temppoint1-1] == 0.0 or arry[temppoint1-1] == pheight):
                InnerEdge = False
        if (InnerEdge):
            # find abaqus indices
            abqindex1 = temppoint1+1
            abqindex2 = temppoint2+1
            p.PartitionFaceByShortestPath(point1=d[abqindex1], point2=d[abqindex2],
                                          faces=pickedFaces)
            edgeindex = np.append(edgeindex, np.uint64(count))
            count = count + 1
        else:
            edgeindex = np.append(edgeindex, np.uint64(0))
    fied.close()

    if dimension == 3:
        # define boundingbox to pick the entire volume
        BoundBox1[0] = -0.1*pwidth
        BoundBox1[1] = -0.1*pheight
        BoundBox1[2] = -0.1*pdepth
        BoundBox2[0] = 1.1*pwidth
        BoundBox2[1] = 1.1*pheight
        BoundBox2[2] = 1.1*pdepth

        p = mdb.models[model_name].parts[part_name]
        c = p.cells
        pickedCells = c.getByBoundingBox(
            BoundBox1[0], BoundBox1[1], BoundBox1[2], BoundBox2[0], BoundBox2[1], BoundBox2[2])
        alledges, d = p.edges, p.datums

        # create grains = cells
        count = 0
        for indexe in range(0, len(edgeindex)):  # cycle over all edges in tess file
            if (edgeindex[indexe] > 0):  # corresponds to an edge on the upper surface
                temppoint1 = int(edgepoint1[indexe])
                temppoint2 = int(edgepoint2[indexe])
                midx = (arrx[temppoint1-1] + arrx[temppoint2-1])/2.0
                midy = (arry[temppoint1-1] + arry[temppoint2-1])/2.0
                p = mdb.models[model_name].parts[part_name]
                c = p.cells
                pickedCells = c.getByBoundingBox(
                    BoundBox1[0], BoundBox1[1], BoundBox1[2], BoundBox2[0], BoundBox2[1], BoundBox2[2])
                alledges, d = p.edges, p.datums
                # find current edge on the upper surface
                tempedge = alledges.findAt((midx, midy, pdepth))
                sweepEdge = alledges.findAt((pwidth, pheight, 0.4*pdepth))
                p.PartitionCellByExtrudeEdge(line=sweepEdge, cells=pickedCells,
                                             edges=tempedge, sense=REVERSE)


def create_set(model_name='Model-1', part_name='PART-1', grain_prefix='GRAIN_', dimension=2):
    # =============================================================================
    # create grains' sets
    # =============================================================================
    if dimension == 2:
        p = mdb.models[model_name].parts[part_name]
        f = p.faces
        tempgrainnumber = 0
        for face in p.faces:
            tempgrainnumber = tempgrainnumber + 1
            puntoface = f[tempgrainnumber-1].pointOn
            faces = f.findAt(
                ((puntoface[0][0], puntoface[0][1], puntoface[0][2]),))
            p.Set(name=grain_prefix + str(tempgrainnumber), faces=faces)
    elif dimension == 3:
        p = mdb.models[model_name].parts[part_name]
        c = p.cells
        tempgrainnumber = 0
        for cella in p.cells:
            tempgrainnumber = tempgrainnumber + 1
            puntocella = c[tempgrainnumber-1].pointOn
            cells = c.findAt(
                ((puntocella[0][0], puntocella[0][1], puntocella[0][2]),))
            p.Set(name=grain_prefix + str(tempgrainnumber), cells=cells)


def neper2cae(model_name='Model-1',
              part_name='PART-1',
              grain_prefix='GRAIN_',
              parametersfilename='',
              datumpointsfilename='',
              edgesfilename='',
              dimension=2):

    # =============================================================================
    # parameters
    # =============================================================================
    fparam = open(parametersfilename, "r")
    pwidth = float(fparam.readline())
    pheight = float(fparam.readline())
    pdepth = float(fparam.readline())
    if dimension == 2:
        pdepth = 0
    Ngrains = int(fparam.readline())
    caepath = fparam.readline()
    fparam.close()

    # =============================================================================
    # Mdb
    # =============================================================================
    executeOnCaeStartup()

    Mdb()

    create_sketch_base(model_name=model_name, width=pwidth,
                       height=pheight, center=[pwidth/2.0, pheight/2.0])

    create_part(model_name=model_name,
                part_name=part_name,
                dimension=dimension,
                pdepth=pdepth,
                pwidth=pwidth,
                pheight=pheight,
                datumpointsfilename=datumpointsfilename,
                edgesfilename=edgesfilename)

    create_set(model_name=model_name,
               part_name=part_name,
               grain_prefix=grain_prefix,
               dimension=dimension)

    mdb_name = r'mesh.cae'
    mdb.saveAs(pathName=mdb_name)



from ..Material.Material import Material
from ..Model.ModelBase import ModelBase


class MaterialModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    Notes
    -----
    This object can be accessed by:

    .. code-block:: python

        mdb.models[name]

    """

    def Material(self, name: str, description: str = '', materialIdentifier: str = ''):
        """This method creates a Material object.

        Notes
        -----
            This function can be accessed by:
            
            .. code-block:: python
            
                mdb.models[name].Material
        
        Parameters
        ----------
        name
            A String specifying the name of the new material.
        description
            A String specifying user description of the material. The default value is an empty
            string.
        materialIdentifier
            A String specifying material identifier for customer use. The default value is an empty
            string.

        Returns
        -------
            A Material object.
        """
        self.materials[name] = material = Material(name, description, materialIdentifier)
        return material



from abaqusConstants import *
from .Material import Material

""" This command evaluates the behavior of a hyperelastic material under standard test 
conditions. 

Notes
-----
    - from material import evaluateMaterial

"""


def evaluateMaterial(material: Material, simulationName: str, dataSource: SymbolicConstant = None,
                     strainEnergyPotentials: SymbolicConstant = None, marlowData: SymbolicConstant = None,
                     marlowDataType: SymbolicConstant = None, testDataTypes: SymbolicConstant = None,
                     uniaxialStrainRange: float = None, biaxialStrainRange: float = None,
                     planarStrainRange: float = None, volumeRatioRange: float = None,
                     simpleShearStrainRange: float = None, viscoDataSource: SymbolicConstant = None,
                     viscoTestDataTypes: SymbolicConstant = None, relaxationTime: float = None,
                     creepTime: float = None):
    """This method evaluates the behavior of a hyperelastic material under standard test
    conditions.

    Notes
    -----
        This function can be accessed by:
        
        .. code-block:: python
        
            evaluateMaterial

    Parameters
    ----------
    material
        A Material object. 
    simulationName
        A String specifying the name to be used for the material evaluation simulation. 
    dataSource
        A SymbolicConstant specifying whether test data or coefficients should be used for the 
        material definition in the unit element tests. Possible values are TEST_DATA or 
        COEFFICIENTS. 
    strainEnergyPotentials
        A sequence of SymbolicConstants specifying for which material models the material is to 
        be evaluated. Possible values are POLY_N1, POLY_N2, POLY_N3, POLY_N4, POLY_N5, POLY_N6, 
        OGDEN_N1, OGDEN_N2, OGDEN_N3, OGDEN_N4, OGDEN_N5, OGDEN_N6, REDUCED_POLY_N1, 
        REDUCED_POLY_N2, REDUCED_POLY_N3, REDUCED_POLY_N4, REDUCED_POLY_N5, REDUCED_POLY_N6, 
        ARRUDA_BOYCE, VAN_DER_WAALS, YEOH, MOONEY_RIVLIN, and NEO_HOOKE.Note:The options 
        POLY_N3, POLY_N4, POLY_N5, and POLY_N6 are valid only if the material was defined by 
        providing coefficients of the strain energy potential. 
    marlowData
        None or a sequence of SymbolicConstants specifying the types of test data to be included 
        in the material definition of the Marlow material that is being evaluated. Possible 
        values are UNIAXIAL, BIAXIAL, PLANAR, or VOLUMETRIC. The default value is None. 
    marlowDataType
        None or a SymbolicConstant specifying the input data type for the Marlow material model. 
        Possible values are TENSION, COMPRESSION, or BOTH. 
    testDataTypes
        A sequence of SymbolicConstants specifying the types of test data to be included in the 
        material definition of the material being evaluated. Possible values are UNIAXIAL, 
        BIAXIAL, PLANAR, and VOLUMETRIC. 
    uniaxialStrainRange
        A tuple of Floats specifying minimum and maximum nominal strains to be applied in the 
        uniaxial tension test. 
    biaxialStrainRange
        A tuple of Floats specifying the minimum and maximum nominal strains to be applied in 
        the biaxial tension test. 
    planarStrainRange
        A tuple of Floats specifying the minimum and maximum nominal strains to be applied in 
        the planar test. The planar test is equivalent to a pure shear test. 
    volumeRatioRange
        A tuple of Floats specifying the minimum and maximum compressive volume ratio. 
    simpleShearStrainRange
        A tuple of Floats specifying the minimum and maximum nominal strains to be applied in 
        the simple shear test. 
    viscoDataSource
        None or a SymbolicConstant specifying whether test data or coefficients should be used 
        for the viscoelastic material definition in the element tests. Possible values are 
        TEST_DATA or COEFFICIENTS. The default value is None. 
    viscoTestDataTypes
        None or a sequence of SymbolicConstants specifying the types of test data to be included 
        in the material definition of the viscoelastic material being evaluated. Possible values 
        are UNIAXIAL, BIAXIAL, PLANAR, or VOLUMETRIC. The default value is None. 
    relaxationTime
        None or a Float specifying the time period for the stress relaxation response mode. The 
        default value is None. 
    creepTime
        None or a Float specifying the time period for the creep response mode. The default 
        value is None. 

    Raises
    ------
        - If *dataSource*=TEST_DATA and *strainEnergyPotentials* contains POLY_N3, POLY_N4, 
        POLY_N5, or POLY_N6: 
          MaterialEvaluationError: POLY_N3, POLY_N4, POLY_N5, or POLY_N6not allowed for 
        *dataSource*=TEST_DATA. 
        - If the material evaluation failed: 
          MaterialEvaluationError: material evaluation failed, see*path to data file*. 
        - If the material type of the material to be evaluated is not hyperelastic: 
          MaterialEvaluationError: Material evaluation is currentlysupported only for 
        hyperelastic materials. 
    """
    pass



# -*- coding: utf-8 -*-
from EulerAngle import EulerAngle


class Grain:
    """The Grain object defines a grain.

    Attributes
    ----------
    name: str
        1st Euler angle
    eulerAngle: EulerAngle
        A :py:class:`~EulerAngle` object.

    Notes
    -----

    """
    name: str = ''
    eulerAngle: EulerAngle = EulerAngle()

    def __init__(self, name: str = '', eulerAngle: EulerAngle = EulerAngle()):
        """The method creates a Grain object.

        Parameters
        ----------
        name: str
            1st Euler angle
        eulerAngle: EulerAngle
            A :py:class:`~EulerAngle` object.

        Notes
        -----

        Returns
        -------
            A Grain object.

        Raises
        ------

        """
        self.name = name
        self.eulerAngle = eulerAngle

    def __str__(self):
        return "{'name': %s, 'eulerAngle': %s}" % (self.name, self.eulerAngle)



class UserDefinedField:
    """The UserDefinedField object redefines field variables at a material point.

    Notes
    -----
        This object can be accessed by:
        
        .. code-block:: python
            
            import material
            mdb.models[name].materials[name].userDefinedField
            import odbMaterial
            session.odbs[name].materials[name].userDefinedField

    The corresponding analysis keywords are:
        - USER DEFINED FIELD

    """

    def __init__(self):
        """This method defines a UserDefinedField object.

        Notes
        -----
            This function can be accessed by:
            
            .. code-block:: python
            
                mdb.models[name].materials[name].UserDefinedField
                session.odbs[name].materials[name].UserDefinedField

        Returns
        -------
            A UserDefinedField object.
        """
        pass



class Depvar:
    """The Depvar object specifies solution-dependent state variables.

    Notes
    -----
        This object can be accessed by:
        
        .. code-block:: python
            
            import material
            mdb.models[name].materials[name].depvar
            import odbMaterial
            session.odbs[name].materials[name].depvar

    The corresponding analysis keywords are:
        - DEPVAR

    """

    def __init__(self, deleteVar: int = 0, n: int = 0):
        """This method creates a Depvar object.

        Notes
        -----
            This function can be accessed by:
            
            .. code-block:: python
            
                mdb.models[name].materials[name].Depvar
                session.odbs[name].materials[name].Depvar
        
        Parameters
        ----------
        deleteVar
            An Int specifying the state variable number controlling the element deletion flag. The 
            default value is 0.This argument applies only to Abaqus/Explicit analyses. 
        n
            An Int specifying the number of solution-dependent state variables required at each 
            integration point. The default value is 0. 

        Returns
        -------
            A Depvar object. 

        Raises
        ------
        RangeError
        """
        pass

    def setValues(self):
        """This method modifies the Depvar object.

        Raises
        ------
        RangeError
        """
        pass



# -*- coding: utf-8 -*-


def SymbolicConstant(in_str):
    """
    :param in_str:
    :return:
    """
    # type: "(str) -> str"
    in_str = in_str
    return in_str


DEG = SymbolicConstant('DEG')
RAD = SymbolicConstant('RAD')


# Variables with simple values

FALSE = False

OFF = 0

ON = 1

TRUE = True


# -*- coding: utf-8 -*-
import os
import numpy as np
from neperparser import NeperParser

# main function
# write here input parameters
# dimensions of the parallelepiped domain
width = 100
height = 100
depth = 1
# number of grains in neper generated polycrystal
# note that the number of grains in abaqus could be lower
# because only the ones intersecting the upper surface are considered
# for the columnar grain structure
Ngrains = 10*10

# execute neper
# custom neper options can be added to this part of the code
# grain growth morphology is used here
os.getcwd()
comando = "neper -T -n " + str(Ngrains)
comando = comando + " -dim 2"
comando = comando + " -domain 'square(" + str(width) + "," + str(height) + ")'"
comando = comando + " -morpho 'gg'"
os.system(comando)

# name of the tess file generated
# may change in future versions of Neper
tessfilename = "n" + str(Ngrains) + "-id1"

# parse neper .tess file
nepermodel = NeperParser(tessfilename)

nepermodel.ReadEulerAngles()
nepermodel.ReadVertices()
nepermodel.ReadEdges()

# report parameters to file that will be read by abaqus script
fparam = open("Parameters.txt", "w")

fparam.write(str(width) + '\n')
fparam.write(str(height) + '\n')
fparam.write(str(depth) + '\n')
fparam.write(str(Ngrains) + '\n')
fparam.write(os.getcwd())

fparam.close()



# -*- coding: utf-8 -*-
import numpy as np
try:
    import pandas as pd
except ImportError:
    print("No module pandas.")
from euler_io import rot_matrix, read_euler


def read_parameters(filename='parameters_AlSi10Mg.xlsx', constants=208, temperature=20):
    """
    Read parameters from the excel file.
    :param filename:
    :param constants:
    :param temperature:
    :return:
    """
    parameters = np.zeros(constants)

    for sheet_name in ['parameters']:
        df = pd.read_excel(filename, sheet_name=sheet_name)

    for i in range(len(df['Number'])):
        parameters[df['Number'][i] - 1] = df[temperature][i]

    return parameters


def write_parameters(parameters=[], filename='', format=''):
    if format == 'txt':
        np.savetxt(filename, parameters)
    if format == 'npy':
        np.save(filename, parameters)


def get_grains_parameters(parameters=[], grains_euler=[]):
    """
    Create parameters for grains.
    :param parameters:
    :param grains_euler:
    :return:
    """
    grains_parameters = {}

    grain_id = grains_euler.keys()

    for i in grain_id:
        para = parameters.copy()

        phi1 = grains_euler[i][0]
        Phi = grains_euler[i][1]
        phi2 = grains_euler[i][2]

        rot = rot_matrix(phi1, Phi, phi2)

        para[57 - 1] = rot[0, 0]
        para[58 - 1] = rot[0, 1]
        para[59 - 1] = rot[0, 2]
        para[65 - 1] = rot[1, 0]
        para[66 - 1] = rot[1, 1]
        para[67 - 1] = rot[1, 2]

        grains_parameters[i] = para

    return grains_parameters


def parameters_to_input(grains_parameters=[], grain_prefix='GRAIN_', constants=208, depvar=1000):
    """
    Write parameters to the input file.
    :param grains_parameters:
    :param grain_prefix:
    :param constants:
    :param depvar:
    :return:
    """
    grain_id = grains_parameters.keys()

    outfile = open('materials.inp', 'w')

    outfile.writelines('**\n')
    outfile.writelines('** MATERIALS\n')
    outfile.writelines('**\n')

    for i in grain_id:
        parameters = grains_parameters[i]

        grain_name = grain_prefix + str(i)

        outfile.writelines('*Material, name=%s\n' % grain_name)
        outfile.writelines('*Depvar\n')
        outfile.writelines('%s,\n' % depvar)
        outfile.writelines('*User Material, constants=%s\n' % constants)

        for i in range(constants / 8):
            outfile.writelines('%16f,%16f,%16f,%16f,%16f,%16f,%16f,%16f\n' % (
                parameters[i * 8],
                parameters[i * 8 + 1],
                parameters[i * 8 + 2],
                parameters[i * 8 + 3],
                parameters[i * 8 + 4],
                parameters[i * 8 + 5],
                parameters[i * 8 + 6],
                parameters[i * 8 + 7]))

    outfile.close()


if __name__ == "__main__":
    grains_euler = read_euler(filename='euler.csv')
    parameters = read_parameters(filename='parameters_AlSi10Mg.xlsx',
                                 constants=208,
                                 temperature=20)
    grains_parameters = get_grains_parameters(parameters=parameters,
                                              grains_euler=grains_euler)
    parameters_to_input(grains_parameters=grains_parameters,
                        grain_prefix='GRAIN_',
                        constants=208,
                        depvar=1000)
    write_parameters(parameters=parameters,
                     filename='parameters.txt',
                     format='txt')
    write_parameters(parameters=parameters,
                     filename='parameters.npy',
                     format='npy')

    p = np.load('parameters.npy')
    print(p)
    p = np.loadtxt('parameters.txt')
    print(p)



# -*- coding: utf-8 -*-
# class representing neper tess file
# and functions for parsing

class NeperParser:

	# prefissofile is the prefix of neper .tess file
	def __init__(self,prefissofile):
		self.prefissofile = prefissofile
		self.filename = prefissofile + '.tess'

	# read Euler angles from file
	def ReadEulerAngles(self):
		fid = open(self.filename,'r')
		euleranglesfilename = self.prefissofile + '-eulerangles.tess'
		fout = open(euleranglesfilename,'w')

		flagcheckekeyword = False # e keyword after *ori
		flagprintnextline = False # next line should be printed or not

		for line in fid:
			if (flagprintnextline):
				if '*' in line: # end of Euler angles
					flagprintnextline = False
				else:
					fout.write(line)
			if (flagcheckekeyword):
				flagprintnextline = True
				flagcheckekeyword = False
			if '*ori' in line: # this is the Euler angles keyword
				flagcheckekeyword = True

		fid.close()
		fout.close()

		return 1

	# read vertices that will become datum points for abaqus
	def ReadVertices(self):
		fid = open(self.filename,'r')
		datumpointsfilename = self.prefissofile + '-datumpoints.tess'
		fout = open(datumpointsfilename,'w')

		flagprintnextline = False # next line should be printed or not
		flagNvertices = False # number of vertices is reported after vertices keyword

		for line in fid:
			if (flagprintnextline):
				if '*' in line: # end of vertices
					flagprintnextline = False
				else:
					fout.write(line)
			if (flagNvertices):
				nverttemp = line.split()
				print('Number of vertices in tess file:' + '\n')
				print(int(nverttemp[0]))
				flagNvertices = False
				flagprintnextline = True
			if '**vertex' in line: # this is the vertices keyword
				flagNvertices = True

		fid.close()
		fout.close()

		return 1

	# read edges that will be used by abaqus to partition face 
	def ReadEdges(self):
		fid = open(self.filename,'r')
		edgesfilename = self.prefissofile + '-edges.tess'
		fout = open(edgesfilename,'w')

		flagprintnextline = False # next line should be printed or not
		flagNedges = False # number of edges is reported after edge keyword

		for line in fid:
			if (flagprintnextline):
				if '*' in line: # end of edges
					flagprintnextline = False
				else:
					fout.write(line)
			if (flagNedges):
				nedgetemp = line.split()
				print('Number of edges in tess file:' + '\n')
				print(int(nedgetemp[0]))
				flagNedges = False
				flagprintnextline = True
			if '**edge' in line:
				flagNedges = True

		fid.close()
		fout.close()

		return 1















